@@ -55,6 +55,14 @@ FIRST_INDEX = ZERO
 SECOND_INDEX = ONE
 LAST_INDEX = -1
 
+#
+# Encodings
+#
+
+ASCII = 'ascii'
+CP1252 = 'cp1252'
+UTF_8 = 'utf-8'
+
 #
 # Functional constants
 #
@@ -66,11 +74,12 @@ MODE_READ_BINARY = 'rb'
 MODE_WRITE = 'w'
 MODE_WRITE_BINARY = 'wb'
 
-UTF8 = 'utf-8'
-
 YES = 'yes'
 NO = 'no'
 
+XML_1_0 = '1.0'
+XML_1_1 = '1.1'
+
 #
 # Return codes
 #
