@@ -11,7 +11,6 @@ with additional functions to read and write text files.
 
 
 import codecs
-import logging
 import os.path
 import shutil
 
@@ -32,7 +31,10 @@ BOM_ASSIGNMENTS = (
 DEFAULT_TARGET_ENCODING = constants.UTF_8
 DEFAULT_FALLBACK_ENCODING = constants.CP1252
 DEFAULT_LINE_ENDING = constants.LF
-DEFAULT_WRITE_MODE = constants.MODE_WRITE_BINARY
+
+SUPPORTED_OUTPUT_LINE_ENDINGS = (
+    constants.LF,
+    constants.CRLF)
 
 #
 # Functions
@@ -40,14 +42,14 @@ DEFAULT_WRITE_MODE = constants.MODE_WRITE_BINARY
 
 
 def to_unicode_and_encoding_name(
-        input_object,
+        bytestring,
         from_encoding=None,
         fallback_encoding=DEFAULT_FALLBACK_ENCODING):
-    """Try to decode the input object to a unicode string
+    """Try to decode bytestring to a unicode string
     and return a tuple containing the conversion result
     and the source encoding name.
 
-    If the input object is not a byte string, a TypeError is raised.
+    If bytestring is not a byte string, a TypeError is raised.
 
     Otherwise, the following algorithm is used:
         - If an explicit input codec was given, decode it using that codec.
@@ -58,36 +60,36 @@ def to_unicode_and_encoding_name(
           in the global DEFAULT_FALLBACK_ENCODING variable but can be
           overridden using the parameter fallback_encoding.
     """
-    if isinstance(input_object, (bytes, bytearray)):
+    if isinstance(bytestring, (bytes, bytearray)):
         if from_encoding:
-            return (input_object.decode(from_encoding),
+            return (bytestring.decode(from_encoding),
                     from_encoding)
         #
         for (bom, encoding) in BOM_ASSIGNMENTS:
-            if input_object.startswith(bom):
-                return (input_object[len(bom):].decode(encoding),
+            if bytestring.startswith(bom):
+                return (bytestring[len(bom):].decode(encoding),
                         encoding)
             #
         #
         try:
-            return (input_object.decode(constants.UTF_8),
+            return (bytestring.decode(constants.UTF_8),
                     constants.UTF_8)
         except UnicodeDecodeError:
-            return (input_object.decode(fallback_encoding),
+            return (bytestring.decode(fallback_encoding),
                     fallback_encoding)
         #
     #
     raise TypeError('This function requires bytes or bytearray as input,'
-                    ' not {0}.'.format(input_object.__class__.__name__))
+                    ' not {0}.'.format(bytestring.__class__.__name__))
 
 
-def to_unicode(input_object,
+def to_unicode(bytestring,
                from_encoding=None,
                fallback_encoding=DEFAULT_FALLBACK_ENCODING):
     """Wrap to_unicode_and_encoding_name(),
     but return the conversion result only."""
     return to_unicode_and_encoding_name(
-        input_object,
+        bytestring,
         from_encoding=from_encoding,
         fallback_encoding=fallback_encoding)[0]
 
@@ -110,16 +112,16 @@ def anything_to_unicode(
 
 
 def to_bytes(
-        input_object,
+        unicode_text,
         to_encoding=DEFAULT_TARGET_ENCODING):
-    """Encode a unicode string to a bytes representation
+    """Encode unicode_text to a bytes representation
     using the provided encoding
     """
-    if isinstance(input_object, str):
-        return input_object.encode(to_encoding)
+    if isinstance(unicode_text, str):
+        return unicode_text.encode(to_encoding)
     #
     raise TypeError('This function requires a unicode string as input,'
-                    ' not {0}.'.format(input_object.__class__.__name__))
+                    ' not {0}.'.format(unicode_text.__class__.__name__))
 
 
 def anything_to_bytes(
@@ -141,11 +143,11 @@ def anything_to_bytes(
     #
 
 
-def to_utf8(input_object):
-    """Encode the input object string to UTF-8
+def to_utf8(unicode_text):
+    """Encode unicode_text to UTF-8
     using this modules's to_bytes() function
     """
-    return to_bytes(input_object, to_encoding=constants.UTF_8)
+    return to_bytes(unicode_text, to_encoding=constants.UTF_8)
 
 
 def anything_to_utf8(
@@ -169,110 +171,75 @@ def fix_double_utf8_transformation(unicode_text,
     This function reverts the effect.
     """
     if wrong_encoding == constants.UTF_8:
-        raise ValueError('This would not have any effect!')
+        raise ValueError('This cannot have any effect!')
     #
-    return to_unicode(to_bytes(unicode_text, to_encoding=wrong_encoding))
+    return to_unicode(
+        to_bytes(
+            unicode_text,
+            to_encoding=wrong_encoding),
+        from_encoding=constants.UTF_8)
 
 
-def lines(input_object,
-          from_encoding=None,
-          fallback_encoding=DEFAULT_FALLBACK_ENCODING,
-          keepends=False):
-    """Iterate over the decoded input object's lines"""
-    for single_line in to_unicode(
-            input_object,
-            from_encoding=from_encoding,
-            fallback_encoding=fallback_encoding).splitlines(keepends=keepends):
-        yield single_line
-    #
-
-
-def read_from_file(input_file,
+def read_from_file(input_file_or_name,
                    from_encoding=None,
                    fallback_encoding=DEFAULT_FALLBACK_ENCODING):
-    """Read input file and return its content as unicode"""
+    """Read input file and return its contents as unicode"""
     try:
-        return to_unicode(input_file.read(),
-                          from_encoding=from_encoding,
-                          fallback_encoding=fallback_encoding)
+        file_contents = input_file_or_name.read()
     except AttributeError:
-        with open(input_file,
+        with open(input_file_or_name,
                   mode=constants.MODE_READ_BINARY) as real_input_file:
-            return read_from_file(real_input_file,
-                                  from_encoding=from_encoding,
-                                  fallback_encoding=fallback_encoding)
+            file_contents = real_input_file.read()
         #
     #
-
-
-def lines_from_file(input_file_or_name,
-                    from_encoding=None,
-                    fallback_encoding=DEFAULT_FALLBACK_ENCODING,
-                    keepends=False):
-    """Iterate over the decoded input file's lines"""
-    decoded_file_content = read_from_file(
-        input_file_or_name,
+    return to_unicode(
+        file_contents,
         from_encoding=from_encoding,
         fallback_encoding=fallback_encoding)
-    for single_line in decoded_file_content.splitlines(keepends=keepends):
-        yield single_line
-    #
 
 
-def prepare_file_output(input_object,
+def prepare_file_output(unicode_content,
                         to_encoding=DEFAULT_TARGET_ENCODING,
-                        from_encoding=None,
-                        fallback_encoding=DEFAULT_FALLBACK_ENCODING,
                         line_ending=DEFAULT_LINE_ENDING):
-    """Return a bytes representation of the input object
+    """Return a bytes representation of unicode_content,
+    with the provided line_ending,
     suitable for writing to a file using mode MODE_WRITE_BINARY.
     """
-    if isinstance(input_object, (tuple, list)):
-        lines_list = []
-        for line in input_object:
-            assert isinstance(line, str)
-            lines_list.append(line)
-        #
+    if line_ending not in SUPPORTED_OUTPUT_LINE_ENDINGS:
+        raise ValueError(
+            'line_ending must be one of {0!r}!'.format(
+                SUPPORTED_OUTPUT_LINE_ENDINGS))
+    #
+    lines_list = []
+    if isinstance(unicode_content, str):
+        lines_list.extend(
+            split.lines_for_reconstruction(unicode_content))
     else:
-        lines_list = list(lines(input_object,
-                                fallback_encoding=fallback_encoding))
-    #
-    return anything_to_bytes(line_ending.join(lines_list),
-                             to_encoding=to_encoding,
-                             from_encoding=from_encoding)
-
-
-# pylint: disable=too-many-arguments; required for versatility
-
-
-def write_to_file(file_name,
-                  input_object,
-                  to_encoding=DEFAULT_TARGET_ENCODING,
-                  from_encoding=None,
-                  fallback_encoding=DEFAULT_FALLBACK_ENCODING,
-                  line_ending=DEFAULT_LINE_ENDING,
-                  write_mode=DEFAULT_WRITE_MODE):
-    """Write the input object or list to the file specified by file_name"""
-    with open(file_name,
-              mode=write_mode) as output_file:
-        output_file.write(
-            prepare_file_output(input_object,
-                                to_encoding=to_encoding,
-                                from_encoding=from_encoding,
-                                fallback_encoding=fallback_encoding,
-                                line_ending=line_ending))
+        for text_block in unicode_content:
+            text_block_lines = split.lines_for_reconstruction(text_block)
+            if not text_block_lines:
+                lines_list.append('')
+                continue
+            #
+            lines_list.extend(text_block_lines)
+        #
     #
+    return to_bytes(
+        line_ending.join(lines_list),
+        to_encoding=to_encoding)
 
 
 def transcode_file(file_name,
                    to_encoding=DEFAULT_TARGET_ENCODING,
                    from_encoding=None,
                    fallback_encoding=DEFAULT_FALLBACK_ENCODING,
-                   line_ending=None,
-                   write_backup_file=True):
-    """Read the input file and transcode it to the specified encoding IN PLACE.
+                   line_ending=None):
+    """Read the input file and transcode it to the specified encoding in place.
+    Raise a ValueError if the detected encoding is the same as the
+    target encoding.
     Preserve original line endings except when specified explicitly.
-    Write a backup file unless explicitly told not to do that.
+    Rename the original file to the original name with the detected
+    encoding name attached.
     """
     with open(file_name,
               mode=constants.MODE_READ_BINARY) as input_file:
@@ -282,27 +249,38 @@ def transcode_file(file_name,
         bytes_content,
         from_encoding=from_encoding,
         fallback_encoding=fallback_encoding)
-    if detected_encoding == to_encoding:
-        logging.warning('File %r is already encoded in %r!',
-                        file_name,
-                        to_encoding)
-        return False
     #
-    if write_backup_file:
-        file_name_root, file_extension = os.path.splitext(file_name)
-        backup_file_name = '{0}.{1}{2}'.format(
-            file_name_root, detected_encoding, file_extension)
-        shutil.move(file_name, backup_file_name)
+    # Check content encoding and raise a ValueError
+    # if the original contents were already encoded
+    # as to_encoding
+    if bytes_content == to_bytes(unicode_content, to_encoding=to_encoding):
+        raise ValueError(
+            'File {0!r} is already encoded in {1!r}!'.format(
+                file_name,
+                to_encoding))
     #
-    if line_ending in (constants.LF, constants.CRLF):
+    # Change line endings if requested
+    if line_ending in SUPPORTED_OUTPUT_LINE_ENDINGS:
         unicode_content = line_ending.join(
             split.lines_for_reconstruction(unicode_content))
+    elif line_ending is not None:
+        raise ValueError(
+            'line_ending – if provided – must be one of {0!r}!'.format(
+                SUPPORTED_OUTPUT_LINE_ENDINGS))
+    #
+    # rename the original file to the backup file name
+    file_name_root, file_extension = os.path.splitext(file_name)
+    backup_file_name = '{0}.{1}{2}'.format(
+        file_name_root, detected_encoding, file_extension)
+    shutil.move(file_name, backup_file_name)
     #
     with open(file_name,
               mode=constants.MODE_WRITE_BINARY) as output_file:
-        output_file.write(to_bytes(unicode_content, to_encoding=to_encoding))
+        output_file.write(
+            to_bytes(
+                unicode_content,
+                to_encoding=to_encoding))
     #
-    return True
 
 
 # vim:fileencoding=utf-8 autoindent ts=4 sw=4 sts=4 expandtab:
